﻿namespace DLC_Tool
{
    using System;
    using System.IO;
    using System.Runtime.Serialization.Formatters.Binary;
    using System.Threading.Tasks;
    using System.Windows.Forms;

    public partial class MainForm : Form
    {
        private static readonly string[] FileOrder = new string[12] { ".TMC", ".TMCL", ".---C", "1.--H", "1.--HL", "2.--H", "2.--HL", "3.--H", "3.--HL", "4.--H", "4.--HL", ".--P" };
        private static DLCData dlcData;
        private static bool newDlc;

        public MainForm()
        {
            InitializeComponent();
        }

        private void ClearCharsUI()
        {
            lbHStyles.Items.Clear();
            lbFiles.Items.Clear();
            tbCosSlot.Clear();
            tbAddTexs.Clear();
            tbHair.Clear();
            tbFace.Clear();
        }

        private void ClearMainUI()
        {
            lbChars.Items.Clear();
            ClearCharsUI();
            tbBCMVer.Text = string.Empty;
            btnSave.Text = "Save";
            btnSave.Enabled = false;
            btnSaveState.Enabled = false;
            btnCharsAdd.Enabled = false;
            btnCharsDelete.Enabled = false;
            btnHStylesAdd.Enabled = false;
            btnHStylesDelete.Enabled = false;
            btnFilesAdd.Enabled = false;
            btnFilesDelete.Enabled = false;
        }

        private void btnOpenBCM_Click(object sender, EventArgs e)
        {
            if (openFileDialogBCM.ShowDialog() == DialogResult.OK)
            {
                OpenFile(openFileDialogBCM.FileName);
            }
        }

        private void MainForm_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Copy;
            }
        }

        private void MainForm_DragDrop(object sender, DragEventArgs e)
        {
            string[] fileNames = (string[])e.Data.GetData(DataFormats.FileDrop);
            Task.Factory.StartNew(() =>
            {
                this.Invoke((Action)delegate
                {
                    if (Path.GetExtension(fileNames[0]).ToLower() == ".bcm")
                    {
                        OpenFile(fileNames[0]);
                    }

                    if (newDlc)
                    {
                        AddFiles(fileNames);
                    }
                });
            });
        }

        private void OpenFile(string fileName)
        {
            try
            {
                newDlc = false;
                ClearMainUI();
                tbSavePath.Text = fileName;
                dlcData = Program.OpenBCM(fileName);
                btnSave.Text = "Save BCM";
                btnSave.Enabled = true;
                btnHStylesAdd.Enabled = true;
                tbCosSlot.ReadOnly = true;
                tbAddTexs.ReadOnly = true;
                tbBCMVer.Text = dlcData.BcmVer.ToString();
                lbChars.BeginUpdate();
                for (int i = 0; i < dlcData.Chars.Count; i++)
                {
                    lbChars.Items.Add(Program.CharNames[dlcData.Chars[i].ID]);
                }

                lbChars.EndUpdate();
                lbChars.SelectedIndex = 0;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void AddFiles(string[] fileNames)
        {
            foreach (string fileName in fileNames)
            {
                if (dlcData.Chars[lbChars.SelectedIndex].Female)
                {
                    for (int i = 0; i < FileOrder.Length; i++)
                    {
                        if (fileName.EndsWith(FileOrder[i], true, null))
                        {
                            dlcData.Chars[lbChars.SelectedIndex].Files[i] = fileName;
                            break;
                        }
                    }
                }
                else
                {
                    for (int i = 0; i < 3; i++)
                    {
                        if (fileName.EndsWith(FileOrder[i], true, null))
                        {
                            dlcData.Chars[lbChars.SelectedIndex].Files[i] = fileName;
                            break;
                        }
                    }

                    if (fileName.EndsWith(FileOrder[11], true, null))
                    {
                        dlcData.Chars[lbChars.SelectedIndex].Files[11] = fileName;
                    }
                }
            }

            ShowFiles(lbChars.SelectedIndex);
        }

        private void lbChars_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbChars.SelectedIndex != -1)
            {
                ShowCharacter(lbChars.SelectedIndex);
            }
        }

        private void ShowCharacter(int charIndex)
        {
            ClearCharsUI();
            tbCosSlot.Text = dlcData.Chars[charIndex].CostumeSlot.ToString();
            tbAddTexs.Text = dlcData.Chars[charIndex].AddTexsCount.ToString();
            if (newDlc)
            {
                if (dlcData.Chars[charIndex].Female)
                {
                    tbAddTexs.ReadOnly = false;
                }
                else
                {
                    tbAddTexs.ReadOnly = true;
                }
            }

            ShowHairstyles(charIndex);
            lbHStyles.SelectedIndex = 0;
            btnHStylesAdd.Enabled = true;
            if (newDlc)
            {
                btnCharsDelete.Enabled = true;
                btnFilesAdd.Enabled = true;
                ShowFiles(charIndex);
            }
        }

        private void ShowHairstyles(int charIndex)
        {
            if (lbHStyles.Items.Count > 0)
            {
                lbHStyles.Items.Clear();
            }

            lbHStyles.BeginUpdate();
            for (int i = 0; i < dlcData.Chars[charIndex].HStyles.Count; i++)
            {
                lbHStyles.Items.Add("Hairstyle " + (char)(65 + i));
            }

            lbHStyles.EndUpdate();
        }

        private void ShowFiles(int charIndex)
        {
            if (lbFiles.Items.Count > 0)
            {
                lbFiles.Items.Clear();
            }

            lbFiles.BeginUpdate();
            foreach (string fileName in dlcData.Chars[charIndex].Files)
            {
                if (fileName != null)
                {
                    lbFiles.Items.Add(fileName);
                }
            }

            lbFiles.EndUpdate();
            btnFilesDelete.Enabled = false;
        }

        private void lbHStyles_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbHStyles.SelectedIndex != -1)
            {
                tbHair.Text = dlcData.Chars[lbChars.SelectedIndex].HStyles[lbHStyles.SelectedIndex].Hair.ToString();
                tbFace.Text = dlcData.Chars[lbChars.SelectedIndex].HStyles[lbHStyles.SelectedIndex].Face.ToString();
                if (lbHStyles.Items.Count > 1)
                {
                    btnHStylesDelete.Enabled = true;
                }
                else
                {
                    btnHStylesDelete.Enabled = false;
                }
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (lbChars.Items.Count == 0)
                {
                    throw new Exception("Characters list is empty!");
                }

                string dlcName = Path.GetFileNameWithoutExtension(tbSavePath.Text);
                if (newDlc)
                {
                    if (tbSavePath.Text == string.Empty)
                    {
                        if (saveFileDialog.ShowDialog() == DialogResult.OK)
                        {
                            tbSavePath.Text = saveFileDialog.FileName;
                        }
                        else
                        {
                            throw new Exception("Please select save path!");
                        }
                    }

                    Directory.CreateDirectory(tbSavePath.Text + @"\data\");
                    Program.SaveBCM(dlcData, tbSavePath.Text + @"\" + dlcName + ".bcm", dlcName);
                    var nameIndexes = Program.SaveBIN(dlcData, tbSavePath.Text + @"\data\" + dlcName + ".bin", dlcName);
                    var fileSizes = Program.SaveLNK(dlcData, tbSavePath.Text + @"\data\" + dlcName + ".lnk", nameIndexes.Count);
                    Program.SaveBLP(tbSavePath.Text + @"\data\" + dlcName + ".blp", nameIndexes, fileSizes);
                    MessageBox.Show("DLC saved!", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    if (tbSavePath.Text == string.Empty)
                    {
                        if (saveFileDialogBCM.ShowDialog() == DialogResult.OK)
                        {
                            tbSavePath.Text = saveFileDialogBCM.FileName;
                        }
                        else
                        {
                            throw new Exception("Please select save path!");
                        }
                    }

                    Program.SaveBCM(dlcData, tbSavePath.Text, dlcName);
                    MessageBox.Show("BCM saved!", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (Exception ex)
            {
                if (ex is OverflowException)
                {
                    MessageBox.Show("Please enter smaller value for DLC name!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void RestrictKeys(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(char.IsDigit(e.KeyChar) || e.KeyChar == 8);
        }

        private void ParseValue(object sender, EventArgs e)
        {
            var inputTB = (TextBox)sender;
            if ((dlcData != null) && (inputTB.Text != string.Empty))
            {
                int inputValue = int.Parse(inputTB.Text);
                if (inputValue > 255)
                {
                    inputValue = 255;
                    inputTB.Text = "255";
                }

                if (inputTB.Name == "tbBCMVer")
                {
                    dlcData.BcmVer = (byte)inputValue;
                }
                else
                {
                    if (lbChars.SelectedIndex != -1)
                    {
                        if (inputTB.Name == "tbCosSlot")
                        {
                            dlcData.Chars[lbChars.SelectedIndex].CostumeSlot = (byte)inputValue;
                        }

                        if (inputTB.Name == "tbAddTexs")
                        {
                            if (inputValue > 4)
                            {
                                inputValue = 4;
                                inputTB.Text = "4";
                            }

                            if (inputValue < 1)
                            {
                                inputValue = 1;
                                inputTB.Text = "1";
                            }

                            dlcData.Chars[lbChars.SelectedIndex].AddTexsCount = (byte)inputValue;
                        }

                        if (lbHStyles.SelectedIndex != -1)
                        {
                            if (inputTB.Name == "tbHair")
                            {
                                dlcData.Chars[lbChars.SelectedIndex].HStyles[lbHStyles.SelectedIndex].Hair = (byte)inputValue;
                            }

                            if (inputTB.Name == "tbFace")
                            {
                                dlcData.Chars[lbChars.SelectedIndex].HStyles[lbHStyles.SelectedIndex].Face = (byte)inputValue;
                            }
                        }
                    }
                }
            }
        }

        private void btnHStylesAdd_Click(object sender, EventArgs e)
        {
            dlcData.Chars[lbChars.SelectedIndex].HStyles.Add(new Hairstyle(1, 1));
            ShowHairstyles(lbChars.SelectedIndex);
            lbHStyles.SelectedIndex = lbHStyles.Items.Count - 1;
        }

        private void btnHStylesDelete_Click(object sender, EventArgs e)
        {
            dlcData.Chars[lbChars.SelectedIndex].HStyles.RemoveAt(lbHStyles.SelectedIndex);
            ShowHairstyles(lbChars.SelectedIndex);
            btnHStylesDelete.Enabled = false;
        }

        private void btnNewDLC_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                newDlc = true;
                ClearMainUI();
                tbSavePath.Text = saveFileDialog.FileName;
                dlcData = new DLCData();
                btnSave.Text = "Save DLC";
                btnSave.Enabled = true;
                btnSaveState.Enabled = true;
                btnCharsAdd.Enabled = true;
                tbCosSlot.ReadOnly = false;
                tbAddTexs.ReadOnly = false;
                dlcData.BcmVer = 9;
                tbBCMVer.Text = "9";
            }
        }

        private void btnCharsAdd_Click(object sender, EventArgs e)
        {
            contextMenuStrip.Show(Cursor.Position.X, Cursor.Position.Y);
        }

        private void AddCharacter(object sender, EventArgs e)
        {
            var inputMI = (ToolStripMenuItem)sender;
            var newChar = new Character();
            newChar.ID = byte.Parse(inputMI.Name.Substring(2));
            if (Program.FemaleIDs.Contains(newChar.ID))
            {
                newChar.Female = true;
            }

            newChar.CostumeSlot = 0;
            newChar.AddTexsCount = 1;
            newChar.HStyles.Add(new Hairstyle(1, 1));
            dlcData.Chars.Add(newChar);
            lbChars.Items.Add(Program.CharNames[newChar.ID]);
            lbChars.SelectedIndex = lbChars.Items.Count - 1;
            btnCharsDelete.Enabled = true;
            btnFilesAdd.Enabled = true;
        }

        private void btnCharsDelete_Click(object sender, EventArgs e)
        {
            dlcData.Chars.RemoveAt(lbChars.SelectedIndex);
            lbChars.Items.RemoveAt(lbChars.SelectedIndex);
            ClearCharsUI();
            btnCharsDelete.Enabled = false;
            btnHStylesAdd.Enabled = false;
            btnFilesAdd.Enabled = false;
            btnFilesDelete.Enabled = false;
        }

        private void btnFilesAdd_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                AddFiles(openFileDialog.FileNames);
            }
        }

        private void btnFilesDelete_Click(object sender, EventArgs e)
        {
            foreach (string selectedFile in lbFiles.SelectedItems)
            {
                for (int i = 0; i < FileOrder.Length; i++)
                {
                    if (selectedFile.EndsWith(FileOrder[i], true, null))
                    {
                        dlcData.Chars[lbChars.SelectedIndex].Files[i] = null;
                        break;
                    }
                }
            }

            ShowFiles(lbChars.SelectedIndex);
        }

        private void lbFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lbFiles.SelectedIndex != -1)
            {
                btnFilesDelete.Enabled = true;
            }
        }

        private void btnSaveState_Click(object sender, EventArgs e)
        {
            try
            {
                if (saveFileDialogState.ShowDialog() == DialogResult.OK)
                {
                    dlcData.SavePath = tbSavePath.Text;
                    var bf = new BinaryFormatter();
                    using (var fs = new FileStream(saveFileDialogState.FileName, FileMode.Create, FileAccess.Write, FileShare.Read))
                    {
                        bf.Serialize(fs, dlcData);
                    }

                    MessageBox.Show("Current state saved!", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnLoadState_Click(object sender, EventArgs e)
        {
            try
            {
                if (openFileDialogState.ShowDialog() == DialogResult.OK)
                {
                    var bf = new BinaryFormatter();
                    using (var fs = new FileStream(openFileDialogState.FileName, FileMode.Open, FileAccess.Read, FileShare.Read))
                    {
                        dlcData = (DLCData)bf.Deserialize(fs);
                    }

                    newDlc = true;
                    ClearMainUI();
                    tbSavePath.Text = dlcData.SavePath;
                    btnSave.Text = "Save DLC";
                    btnSave.Enabled = true;
                    btnSaveState.Enabled = true;
                    btnCharsAdd.Enabled = true;
                    tbCosSlot.ReadOnly = false;
                    tbAddTexs.ReadOnly = false;
                    tbBCMVer.Text = dlcData.BcmVer.ToString();
                    lbChars.BeginUpdate();
                    for (int i = 0; i < dlcData.Chars.Count; i++)
                    {
                        lbChars.Items.Add(Program.CharNames[dlcData.Chars[i].ID]);
                    }

                    lbChars.EndUpdate();
                    if (lbChars.Items.Count > 0)
                    {
                        lbChars.SelectedIndex = 0;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}